@@ -24,7 +24,6 @@ from nuitka.__past__ import iter_modules, unicode
 from nuitka.containers.Namedtuples import makeNamedtupleClass
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import (
-    decodeDataFileTags,
     makeIncludedDataDirectory,
     makeIncludedDataFile,
     makeIncludedEmptyDirectory,
@@ -52,7 +51,7 @@ from nuitka.Options import (
     isOnefileMode,
     isOnefileTempDirMode,
     isStandaloneMode,
-    shallCreateAppBundle,
+    shallMakeDll,
     shallMakeModule,
     shallShowExecutedCommands,
 )
@@ -82,8 +81,12 @@ from nuitka.utils.Execution import (
 from nuitka.utils.FileOperations import (
     changeFilenameExtension,
     getFileContents,
+    getReportSourceReference,
+)
+from nuitka.utils.Importing import (
+    getExtensionModuleSuffix,
+    isBuiltinModuleName,
 )
-from nuitka.utils.Importing import getSharedLibrarySuffix, isBuiltinModuleName
 from nuitka.utils.ModuleNames import (
     ModuleName,
     makeTriggerModuleName,
@@ -157,6 +160,7 @@ def _getEvaluationContext():
             "onefile": isOnefileMode(),
             "onefile_cached": not isOnefileTempDirMode(),
             "module_mode": shallMakeModule(),
+            "dll_mode": shallMakeDll(),
             "deployment": isDeploymentMode(),
             # Version information
             "company": getCompanyName(),
@@ -229,10 +233,12 @@ def _getEvaluationContext():
         _context_dict["python3_or_higher"] = python_version >= 0x300
 
         if not isNuitkaPython():
-            _context_dict["extension_std_suffix"] = getSharedLibrarySuffix(
+            _context_dict["extension_std_suffix"] = getExtensionModuleSuffix(
                 preferred=True
             )
-            _context_dict["extension_suffix"] = getSharedLibrarySuffix(preferred=False)
+            _context_dict["extension_suffix"] = getExtensionModuleSuffix(
+                preferred=False
+            )
 
     return _context_dict
 
@@ -736,6 +742,16 @@ class NuitkaPluginBase(getMetaClassBase("Plugin", require_slots=False)):
             None
         """
 
+    def onCompilationStartChecks(self):
+        """The compilation is setup, locating modules if expected to work.
+
+        Args:
+            None
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use
+
     def onModuleInitialSet(self):
         """Provide extra modules to the initial root module set.
 
@@ -793,8 +809,7 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
                     }
                 )
 
-    @staticmethod
-    def locateModule(module_name):
+    def locateModule(self, module_name):
         """Provide a filename / -path for a to-be-imported module.
 
         Args:
@@ -972,24 +987,6 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
             tags=tags,
         )
 
-    def makeIncludedAppBundleResourceFile(
-        self, source_path, dest_path, reason, tags=""
-    ):
-        tags = decodeDataFileTags(tags)
-        tags.add("framework_resource")
-
-        assert isMacOS() and shallCreateAppBundle()
-
-        # The default dest path root is the "Contents" folder
-        dest_path = os.path.join("..", "Resources", dest_path)
-
-        return self.makeIncludedDataFile(
-            source_path=source_path,
-            dest_path=dest_path,
-            reason=reason,
-            tags=tags,
-        )
-
     def makeIncludedGeneratedDataFile(self, data, dest_path, reason, tags=""):
         return makeIncludedGeneratedDataFile(
             data=data, dest_path=dest_path, reason=reason, tracer=self, tags=tags
@@ -1223,6 +1220,11 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
             )
         )
 
+    @staticmethod
+    def getReportSourceReference(source_ref):
+        """Format a source reference suitable for user output."""
+        return getReportSourceReference(source_ref)
+
     def getExtraCodeFiles(self):
         """Add extra code files to the compilation.
 
